from google.cloud import translate_v2 as translate
from google.cloud import language_v1 as lang
from typing import Dict
from google.cloud.language_v1.types.language_service import AnalyzeEntitiesResponse
import os

class NewsNLP(object):

    def __init__(self, cred_path: str):

        if not os.path.exists(cred_path):
            raise FileNotFoundError(f"Please provide a valid Google Credentials.")

        os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = cred_path
        self.translate_client = translate.Client()
        self.client = lang.LanguageServiceClient()
        self.type_ = lang.Document.Type.PLAIN_TEXT
        self.encoding_type = lang.EncodingType.UTF8
        self.document: Dict = {}
        self.detected_language: str = ""

    def __get_document__(self) -> Dict:
        return self.document

    def __get_encoding__(self):
        return self.encoding_type

    def google_cloud_tag_entities(self, news_article: str, doc_lang: str = 'en',
                                  detect_lang: bool = False) -> AnalyzeEntitiesResponse:

        if not news_article:
            raise ValueError("Please provide a valid news article.")

        if detect_lang:
            self.detected_language: str = self.detect_language(news_article=news_article)

        self.document = {"content": news_article,
                         "type_": self.type_,
                         "language": self.detected_language if detect_lang else doc_lang
                         }

        response = self.client.analyze_entities(request={'document': self.document,
                                                         'encoding_type': self.encoding_type})

        return response

    def detect_language(self, news_article: str) -> str:

        if not news_article:
            raise AttributeError("Please provide a valid news article.")

        return self.translate_client.detect_language(news_article).get('language', '')


import pytest
from unittest import TestCase
from acme_news.utils.tools import get_synonyms
import os

class TestUtils(TestCase):
    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_synonyms(self):
        response = get_synonyms(word='eat')
        print(response)
        return self.assertEqual(31, response.__len__())


#!/usr/bin/env python
import os
import re
import gzip
from typing import List, Dict
from uuid import uuid4
from acme_news.browser.acme_browser import ACMEBrowser

from googleapiclient.discovery import build
from googleapiclient.http import MediaFileUpload
from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow

from nltk import sent_tokenize, ne_chunk, pos_tag, word_tokenize
from nltk.sentiment.vader import SentimentIntensityAnalyzer
import pandas as pd

def tarball(file_name: str, output_path: str = "") -> Dict:
    if not output_path:
        output_path: str = re.sub(r"^./", "", file_name).rstrip("/").replace("/", "_")

    tar_ball: str = f"{output_path}_news_middle_east_regions_{str(uuid4())}.tar.gz"

    joined_contents = None
    current_content = None

    if os.path.isdir(file_name):
        contents: List = []
        for root, dirs, content_files in os.walk(file_name):
            for csv_file in content_files:
                current_file_path: str = os.path.join(root, csv_file)
                if os.path.getsize(current_file_path) > 0:
                    contents.append(open(current_file_path, 'r').read())

        joined_contents = '\n'.join(contents).encode('utf-8')

    if os.path.isfile(file_name):
        try:
            current_content = open(file_name, 'r').read().encode('utf-8')

        except IOError as e:
            raise IOError(f"Unable to open {file_name}") from e

    with gzip.open(tar_ball, 'w') as f:
        f.write(joined_contents if joined_contents else current_content)
    f.close()

    return {'file': tar_ball,
            'status': "created" if os.path.exists(tar_ball) else "failed"
           }


def upload_to_drive(cred_file: str,
                   file_name: str,
                   api_scope: str = "https://www.googleapis.com/auth/drive",
                   mimetype: str = 'application/gzip') -> Dict:

    if not os.path.exists(cred_file):
        raise FileNotFoundError(f"Unable to find the following file {cred_file}")

    if not os.path.exists(file_name):
        raise FileNotFoundError(f"Unable to find the following file {file_name}")

    creds = None
    try:
        creds = Credentials.from_authorized_user_file(cred_file)
    except:
        pass

    if not creds and not creds.valid:
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())

        else:
            flow = InstalledAppFlow.from_client_secrets_file(cred_file, api_scope)
            creds = flow.run_local_server(port=58887)

        with open(os.path.join(os.path.expanduser("~/"), cred_file)) as f:
            f.write(creds.to_json())

    google_drive = build('drive', 'v3', credentials=creds)
    media = MediaFileUpload(file_name, mimetype=mimetype, resumable=True)
    file_metadata: Dict = {'name': file_name,
                           'mimeType': 'application/gzip'
                           }
    response = google_drive.files().create(body=file_metadata, media_body=media, fields='id').execute()

    return response

def get_sentiment_scores(news_article: str) -> Dict:
    if not news_article:
        raise ValueError("Please provide a valid news article.")

    return pd.DataFrame([SentimentIntensityAnalyzer().polarity_scores(sentence) for sentence in sent_tokenize(news_article)]).mean().to_dict()

def tag_entity(news_article: str) -> Dict:
    if not news_article:
        raise ValueError("Please provide a valid news article.")

    entities = ne_chunk(pos_tag(word_tokenize(news_article)))
    tag_entities: Dict = {entity.label(): entity[0][0] for entity in entities if hasattr(entity, 'label')}

    return tag_entities

def get_synonyms(word: str) -> List[str]:
    if not word:
        raise ValueError("Please provide the right search keyword.")
    try:
        driver = ACMEBrowser().browser(url=f"https://www.thesaurus.com/misspelling?term={word}")
        return list(filter(lambda synonym: len(word_tokenize(synonym)) == 1,
                           driver.find_element_by_css_selector("div#meanings").text.split("\n")))

    except:
        return []





#!/usr/bin/python
import networkx as nx
import squarify
import matplotlib.pyplot as plt
import seaborn as sns
import os

from pandas.core.frame import DataFrame
from typing import List

def save_graph(path_name: str) -> bool:
    if not path_name:
        raise ValueError("Please provide the path_name.")

    plt.savefig(path_name)
    return os.path.exists(path_name)

def plot_treemap(df: DataFrame,
                 ncols: int,
                 region: str,
                 columns: List[str],
                 dimension: tuple = (75, 15),
                 palette: str = 'flare_r',
                 nrows: int = 1,
                 save: bool = False
                 ):
    if not isinstance(df, DataFrame):
        raise TypeError("Please proivde the right dataframe.")

    if not all([ncols, region, columns, dimension]):
        raise ValueError("Please provide the required parameters such as ncols, region, columns, and dimensions.")

    plt.rcParams['figure.figsize'] = dimension
    fig, axs = plt.subplots(ncols=ncols, nrows=nrows)

    for index, ax in enumerate(axs.ravel()):
        organizations, counts = zip(
            *df.query(f"region_name == '{region}'")[columns[index]].value_counts().to_dict().items())
        squarify.plot(sizes=counts,
                      label=organizations,
                      color=sns.color_palette(palette=palette,
                                              n_colors=50
                                              ),
                      text_kwargs={'color': 'white',
                                   'weight': 'bold'},
                      ax=ax
                      )
        title: str = f"{region} news articles: {columns[index]}  Keywords".title()
        ax.set_title(title,
                     fontweight='bold',
                     fontsize=22)

        ax.invert_yaxis()
        ax.axis('off')

    if save:
        file_path: str = f"{title.replace(' ', '_')}.jpg"
        status: bool = save_graph(file_path)
        return {'file': file_path,
                'status': status
                }

def draw_spring_network(nodes: List,
                        edges: List,
                        title: str,
                        scale: int = 18,
                        k: int = 0.29,
                        node_color: str = 'red',
                        node_size: int = 100,
                        figsize: tuple = (25, 12),
                        font_size: int = 12,
                        edge_color: str = "grey",
                        save: bool = False
                        ):

    if not all([nodes, edges, title]):
        raise ValueError("Please provide the following parameters: nodes, edges, and title")

    nx_graph = nx.Graph()
    nx_graph.add_nodes_from(nodes)

    for entity_attr in edges:
        nx_graph.add_edge(entity_attr[0], entity_attr[1], weight=entity_attr[-1])

    pos = nx.spring_layout(nx_graph, scale=scale, k=k)

    plt.rcParams['figure.figsize'] = figsize
    nx.draw_networkx_nodes(nx_graph, pos, node_color=node_color, node_size=node_size)
    nx.draw_networkx_labels(nx_graph, pos, font_size=font_size)
    nx.draw_networkx_edges(nx_graph, pos, edge_color=edge_color)
    plt.axis('off')
    plt.title(title, fontsize=26, fontweight='bold')

    if save:
        file_path: str = f"{title.replace(' ', '_')}.jpg"
        status: bool = save_graph(file_path)
        return {'file': file_path,
                'status': status}
